"""Output the licenses used by dependencies and check if these are compatible
with the project license
"""
from __future__ import annotations

import subprocess
import argparse
from sys import exit as sysexit, stdout
import warnings

import requirements

from licensecheck import packageinfo, license_matrix, formatter
from licensecheck.packagecompat import PackageCompat

stdout.reconfigure(encoding="utf-8")


def _doSysExec(command: str) -> tuple[int, str]:
	"""execute a command and check for errors

	Args:
		command (str): commands as a string

	Raises:
		RuntimeWarning: throw a warning should there be a non exit code
	"""
	with subprocess.Popen(command, shell=True, stdout=subprocess.PIPE,
	stderr=subprocess.STDOUT, encoding="utf-8", errors="ignore") as process:
		out = process.communicate()[0]
		exitCode = process.returncode
	return exitCode, out


def getdepsLicenses() -> list[PackageCompat]:
	"""Get a list of packages with package compatibility

	Returns:
		list[PackageCompat]: list of packages (python dicts)
	"""
	# Is poetry installed?
	poetryInstalled = True
	if _doSysExec("poetry -h")[0] != 0:
		poetryInstalled = False
		warnings.warn(RuntimeWarning("poetry is not on the system path"))
	# Get list of requirements
	reqs = []
	if poetryInstalled:
		# Use poetry show to get dependents of dependencies
		lines = _doSysExec("poetry show")[1].splitlines(False)
		for line in lines:
			parts = line.split()
			reqs.append(parts[0])
	else:
		with open("requirements.txt", 'r') as requirementsTxt:
			for req in requirements.parse(requirementsTxt): # type: ignore
				reqs.append(req.name) # type: ignore
	# Get my license
	myLiceTxt = packageinfo.getMyPackageLicense()
	myLice = license_matrix.licenseType(myLiceTxt)[0]
	# Check it is compatible with packages and add a note
	depsLicenses = []
	for package in packageinfo.getPackages(reqs):
		depLice = license_matrix.licenseType(package["license"])
		depsLicenses.append(PackageCompat(**package,
		license_compat=license_matrix.depCompatibleLice(myLice, depLice)))
	return depsLicenses

FORMAT_HELP = "Output format. One of simple, ansi, json, markdown, csv. default=simple"

def cli() -> None:
	""" cli entry point """
	parser = argparse.ArgumentParser(description=__doc__)
	parser.add_argument("--format", "-f", help=FORMAT_HELP)
	parser.add_argument("--file", "-o",
	help="Filename to write to (omit for stdout)")
	args = parser.parse_args()
	# File
	filename = stdout if args.file is None else open(args.file, "w")
	# Format
	formatMap = {
	"json": formatter.json, "markdown": formatter.markdown, "csv": formatter.csv,
	"ansi": formatter.ansi, "simple": formatter.simple}
	if args.format is None:
		print(formatter.simple(getdepsLicenses()), file=filename)
	elif args.format in formatMap:
		print(formatMap[args.format](getdepsLicenses()), file=filename)
	else:
		print(FORMAT_HELP)
		sysexit(1)



"""
Take our package compat dictionary and give things a pretty format

```json
{
	name: str
	version: str
	namever: str
	size: int
	home_page: str
	author: str
	license: str
	license_compat: bool
}
```

Formats

- markdown
- json
- csv
- ansi
"""
from __future__ import annotations

from io import StringIO
from json import dumps
from csv import writer
import typing

from licensecheck.packagecompat import PackageCompat
logger = None
try:
	from metprint import Logger, LogType
	logger = Logger()
except ModuleNotFoundError:
	pass

def markdown(packages: list[PackageCompat],
heading: typing.Optional[str] = None) -> str:
	"""Format to Markdown

	Args:
		packages (list[PackageCompat]): PackageCompats to format
		heading (str, optional): Optional heading to include. Defaults to None.

	Returns:
		str: String to write to a file of stdout
	"""
	if len(packages) == 0:
		return "No packages"

	heading = heading if heading is not None else \
 	"# Packages\nFind a list of packages below"
	strBuf = [heading]
	packages = sorted(packages, key=lambda i: i["name"])

	# Summary Table
	strBuf.append("")
	strBuf.append("|Compatible|Package|\n|:--|:--|")
	for pkg in packages:
		strBuf.append(f"|{pkg['license_compat']}|{pkg['name']}|")
	strBuf.append("")

	# Details
	for pkg in packages:
		strBuf.extend([
		f"## {pkg['namever']}",
		f"\n\n- HomePage: {pkg['home_page']}",
		f"\n- Author: {pkg['author']}",
		f"\n- License: {pkg['license']}",
		f"\n- Compatible: {pkg['license_compat']}",
		f"\n- Size: {pkg['size']}"]) # yapf: disable
	return "\n".join(strBuf) + "\n"


def json(packages: list[PackageCompat],
heading: typing.Optional[str] = None) -> str:
	"""Format to Json

	Args:
		packages (list[PackageCompat]): PackageCompats to format
		heading (str, optional): Optional heading to include. Defaults to None.

	Returns:
		str: String to write to a file of stdout
	"""
	packages = sorted(packages, key=lambda i: i["name"])
	out = {"heading": heading if heading is not None else \
 	"# Packages - Find a list of packages below",
	"packages": packages}
	return dumps(out, indent="\t")


def csv(packages: list[PackageCompat],
heading: typing.Optional[str] = None) -> str:
	"""Format to CSV

	Args:
		packages (list[PackageCompat]): PackageCompats to format
		heading (str, optional): Optional heading to include. Defaults to None.

	Returns:
		str: String to write to a file of stdout
	"""
	packages = sorted(packages, key=lambda i: i["name"])
	output = StringIO()
	csvString = writer(output)
	csvString.writerow([heading if heading is not None else \
 	"# Packages - Find a list of packages below (you may want to delete this line)"])
	csvString.writerow([
	"name", "version", "namever", "home_page", "author", "size", "license",
	"license_compat"])
	for pkg in packages:
		csvString.writerow([
		pkg["name"], pkg["version"], pkg["namever"], pkg["home_page"], pkg["author"],
		pkg["size"], pkg["license"], pkg["license_compat"]])
	return output.getvalue()


def ansi(packages: list[PackageCompat],
heading: typing.Optional[str] = None) -> str:
	"""Format to ansi

	Args:
		packages (list[PackageCompat]): PackageCompats to format
		heading (str, optional): Optional heading to include. Defaults to None.

	Returns:
		str: String to write to a file of stdout
	"""
	# pylint: disable=invalid-name
	BLD = "\033[01m"
	CLS = "\033[00m"
	UL = "\033[04m"
	CB = "\033[36m"
	CG = "\033[32m"

	if len(packages) == 0:
		return f"{BLD}{UL}{CB}No packages{CLS}"

	# pylint: enable=invalid-name
	heading = heading if heading is not None else \
 	f"{BLD}{UL}{CB}PackageCompats{CLS}\n\nFind a list of packages below ordered by severity\n"
	strBuf = [heading]
	packages = sorted(packages, key=lambda i: i["name"])

	# Summary Table
	strBuf.append(f"┌{'─'*10}┬{'─'*30}┐")
	strBuf.append("│Compatible│Package                       │")
	strBuf.append(f"├{'─'*10}┼{'─'*30}┤")
	for pkg in packages:
		strBuf.append(f"│{str(pkg['license_compat']): <10}│{pkg['name'][:30]: <30}│")
	strBuf.append(f"└{'─'*10}┴{'─'*30}┘")
	strBuf.append("")

	# Details
	for pkg in packages:
		strBuf.extend([
		f"{BLD}{UL}{CG}{pkg['namever']}{CLS}",
		f"HomePage: {pkg['home_page']}",
		f"Author: {pkg['author']}",
		f"License: {pkg['license']}",
		f"Compatible: {pkg['license_compat']}",
		f"Size: {pkg['size']}\n"]) # yapf: disable
	return "\n".join(strBuf)


def simple(packages: list[PackageCompat]) -> str:
	"""Format to ansi

	Args:
		packages (list[PackageCompat]): PackageCompats to format

	Returns:
		str: String to write to a file of stdout
	"""
	packages = sorted(packages, key=lambda i: i["name"])

	# Summary Table
	if logger is None:
		strBuf = [f"┌{'─'*10}┬{'─'*20}┬{'─'*20}┐"]
		strBuf.append("│Compatible│Package             │License             │")
		strBuf.append(f"├{'─'*10}┼{'─'*20}┼{'─'*20}┤")
		for pkg in packages:
			strBuf.append(f"│{str(pkg['license_compat']): <10}│{pkg['name'][:20]: <20}│{pkg['license'][:20]: <20}│")
		strBuf.append(f"└{'─'*10}┴{'─'*20}┴{'─'*20}┘")
	else:
		strBuf = [logger.logString(f"┌{'─'*20}┬{'─'*30}┐", LogType.NONE, True)]
		strBuf.append(logger.logString("│Package             │License                       │", LogType.NONE, True))
		strBuf.append(logger.logString(f"├{'─'*20}┼{'─'*30}┤", LogType.NONE, True))
		for pkg in packages:
			strBuf.append(logger.logString(
				f"│{pkg['name'][:20]: <20}│{pkg['license'][:30]: <30}│",
				LogType.SUCCESS if pkg['license_compat'] else LogType.ERROR))
		strBuf.append(logger.logString(f"└{'─'*20}┴{'─'*30}┘", LogType.NONE, True))
	return "\n".join(strBuf)



"""Get information for installed and online packages
"""

from __future__ import annotations

from email.message import Message
from email.parser import FeedParser
from email import message_from_string
import os
import os.path
import typing

from pip._vendor.pkg_resources import Distribution
from pip._internal.utils.misc import get_installed_distributions

import requests
import tomlkit
from tomlkit import items

LICENSE_UNKNOWN = 'UNKNOWN'


class PackageInfo(typing.TypedDict):
	"""PackageInfo type
	"""
	name: str
	version: str
	namever: str
	size: int
	home_page: str
	author: str
	license: str


def getPackagesFromLocal(requirements: list[str]) -> list[PackageInfo]:
	"""Get a list of package info from local files including version, author, and
	the license

	Args:
		requirements (list[str]): [description]

	Returns:
		list[PackageInfo]: [description]
	"""
	# Filter our packages
	pkgsToKeep = []
	pkgs = get_installed_distributions()
	for pkg in pkgs:
		if pkg.project_name in requirements:
			pkgsToKeep.append(pkg)

	pkgInfo = []
	for pkg in pkgsToKeep:
		licenseClassifier = homePage = author = lice = LICENSE_UNKNOWN
		# Try and get metadata
		metadata = None
		if pkg.has_metadata('METADATA'):
			metadata = pkg.get_metadata('METADATA')
		if pkg.has_metadata('PKG-INFO') and metadata is None:
			metadata = pkg.get_metadata('PKG-INFO')
		if metadata is not None:
			message = message_from_string(metadata)
			licenseClassifier = licenseFromClassifierMessage(message)
			feedParser = FeedParser()
			feedParser.feed(metadata)
			parsedMetadata = feedParser.close()
			homePage = parsedMetadata.get("home-page", LICENSE_UNKNOWN)
			author = parsedMetadata.get("author", LICENSE_UNKNOWN)
			lice = parsedMetadata.get("license", LICENSE_UNKNOWN)

		pkgLicense = licenseClassifier if licenseClassifier != LICENSE_UNKNOWN else lice
		pkgInfo.append({
		"name": pkg.project_name,
		"version": pkg.version,
		"namever": str(pkg),
		"home_page": homePage,
		"author": author,
		"size": getModuleSize(pkg),
		"license": pkgLicense}) # yapf: disable
	return pkgInfo


def licenseFromClassifierMessage(message: Message) -> str:
	"""Get license string from a Message of project classifiers

	Args:
		classifiers (Message): Message of classifiers

	Returns:
		str: the license name
	"""
	fromClassifier = LICENSE_UNKNOWN
	licenses = []
	for key, val in message.items():
		if key == 'Classifier' and val.startswith('License'):
			lice = val.split(' :: ')[-1]
			# Through the declaration of 'Classifier: License :: OSI Approved'
			if lice != 'OSI Approved':
				licenses.append(lice)
	if len(licenses) > 0:
		fromClassifier = ', '.join(licenses)
	return fromClassifier


def getPackagesFromOnline(requirements: list[str]) -> list[PackageInfo]:
	"""Get a list of package info from pypi.org including version, author, and
	the license

	Args:
		requirements (list[str]): [description]

	Returns:
		list[PackageInfo]: [description]
	"""
	pkgInfo = []
	for pkg in requirements:
		url = "https://pypi.org/pypi/" + pkg + "/json"
		request = requests.get(url) # type: ignore
		response = request.json() # type: ignore
		info = response["info"]
		licenseClassifier = licenseFromClassifierlist(response["info"]
		["classifiers"])
		pkgLicense = licenseClassifier if licenseClassifier != LICENSE_UNKNOWN else info[
		"license"]
		pkgInfo.append({"name": pkg,
		"version": info["version"],
		"namever": f"{pkg} {info['version']}",
		"home_page": info["home_page"],
		"author": info["author"],
		"size": int(response["urls"][-1]["size"]),
		"license": pkgLicense}) # yapf: disable
	return pkgInfo


def licenseFromClassifierlist(classifiers: list[str]) -> str:
	"""Get license string from a list of project classifiers

	Args:
		classifiers (list[str]): list of classifiers

	Returns:
		str: the license name
	"""
	fromClassifier = LICENSE_UNKNOWN
	licenses = []
	for val in classifiers:
		if val.startswith('License'):
			lice = val.split(' :: ')[-1]
			# Through the declaration of 'Classifier: License :: OSI Approved'
			if lice != 'OSI Approved':
				licenses.append(lice)
	if len(licenses) > 0:
		fromClassifier = ', '.join(licenses)
	return fromClassifier


def getPackages(reqs: list[str]) -> list[PackageInfo]:
	"""Get dependency info

	Args:
		reqs (list[str]): list of dependency names to gather info on

	Returns:
		list[PackageInfo]: list of dependencies
	"""
	localReqs = getPackagesFromLocal(reqs)
	for localReq in localReqs:
		reqs.remove(localReq["name"])
	onlineReqs = getPackagesFromOnline(reqs)
	return localReqs + onlineReqs


def getMyPackageLicense() -> str:
	""" get the pyproject data

	Returns:
		str: license name
	"""
	pyproject = None
	try:
		with open("pyproject.toml") as pyproject:
			pyproject = tomlkit.parse(pyproject.read())
	except FileNotFoundError:
		return input("Enter the project license")
	tool = typing.cast(items.Table, pyproject["tool"])
	metaData = {"classifiers": [], "license": ""}
	if "poetry" in tool:
		metaData = typing.cast(items.Table, tool["poetry"])
	elif "flit" in tool:
		metaData = typing.cast(items.Table,
		typing.cast(items.Table, tool["flit"])["metadata"])
	else:
		return input("Enter the project license")
	licenseClassifier = licenseFromClassifierlist(metaData["classifiers"]) # type: ignore
	if licenseClassifier != LICENSE_UNKNOWN:
		return licenseClassifier
	return str(metaData["license"])


def calcContainer(path: str) -> int:
	"""Get size of installed module from path

	Args:
		path (str): path to the module

	Returns:
		int: size in bytes
	"""
	totalSize = 0
	for dirpath, _dirnames, filenames in os.walk(path, followlinks=True):
		for file in filenames:
			filePointer = os.path.join(dirpath, file)
			totalSize += os.path.getsize(filePointer)
	return totalSize


def getModuleSize(pkg: Distribution) -> int:
	"""Get the size of a given module as an int

	Args:
		pkg (Distribution): package to get the size of

	Returns:
		int: size in bytes
	"""
	pkgDirname = "{}-{}.dist-info".format(pkg.project_name.replace("-", "_"),
	pkg.version)
	path = os.path.join(pkg.location, pkgDirname) # type: ignore
	size = calcContainer(path)
	if size > 0:
		return size
	url = "https://pypi.org/pypi/" + pkg.project_name + "/json"
	request = requests.get(url) # type: ignore
	response = request.json() # type: ignore
	return int(response["urls"][-1]["size"])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['licensecheck']

package_data = \
{'': ['*'], 'licensecheck': ['pypi_licenses/*']}

install_requires = \
['pip>=20.2.3,<21.0.0',
 'requests>=2.24.0,<3.0.0',
 'requirements-parser>=0.2.0,<0.3.0',
 'tomlkit>=0.7.0,<0.8.0']

extras_require = \
{'full': ['metprint>=2020.7,<2021.0']}

entry_points = \
{'console_scripts': ['licensecheck = licensecheck:cli']}

setup_kwargs = {
    'name': 'licensecheck',
    'version': '2020.0.4',
    'description': 'Output the licenses used by dependencies and check if these are compatible with the project license',
    'long_description': '[![Github top language](https://img.shields.io/github/languages/top/FHPythonUtils/LicenseCheck.svg?style=for-the-badge)](../../)\n[![Codacy grade](https://img.shields.io/codacy/grade/.svg?style=for-the-badge)](https://www.codacy.com/gh/FHPythonUtils/LicenseCheck)\n[![Repository size](https://img.shields.io/github/repo-size/FHPythonUtils/LicenseCheck.svg?style=for-the-badge)](../../)\n[![Issues](https://img.shields.io/github/issues/FHPythonUtils/LicenseCheck.svg?style=for-the-badge)](../../issues)\n[![License](https://img.shields.io/github/license/FHPythonUtils/LicenseCheck.svg?style=for-the-badge)](/LICENSE.md)\n[![Commit activity](https://img.shields.io/github/commit-activity/m/FHPythonUtils/LicenseCheck.svg?style=for-the-badge)](../../commits/master)\n[![Last commit](https://img.shields.io/github/last-commit/FHPythonUtils/LicenseCheck.svg?style=for-the-badge)](../../commits/master)\n[![PyPI Downloads](https://img.shields.io/pypi/dm/LicenseCheck.svg?style=for-the-badge)](https://pypi.org/project/LicenseCheck/)\n[![PyPI Version](https://img.shields.io/pypi/v/LicenseCheck.svg?style=for-the-badge)](https://pypi.org/project/LicenseCheck/)\n\n<!-- omit in toc -->\n# LicenseCheck\n\n<img src="readme-assets/icons/name.png" alt="Project Icon" width="750">\n\nOutput the licences used by dependencies and check if these are compatible with\nthe project license\n\n\n## Example Use\n\nSee below for the output if you run `licensecheck` in this directory\n\n```txt\n>> licensecheck\n    |Package             |License                       |\n    |--------------------|------------------------------|\n[+] |certifi             |Mozilla Public License 2.0 (MP|\n[+] |chardet             |GNU Library or Lesser General |\n[+] |idna                |BSD License                   |\n[+] |metprint            |MIT License                   |\n[+] |requests            |Apache Software License       |\n[+] |requirements-parser |BSD License                   |\n[+] |urllib3             |MIT License                   |\n```\n\n### Help\n\n```txt\nusage: __main__.py [-h] [--format FORMAT] [--file FILE]\n\nOutput the licenses used by dependencies and check if these are compatible with the project license\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --format FORMAT, -f FORMAT\n                        Output format. One of simple, ansi, json, markdown, csv. default=simple\n  --file FILE, -o FILE  Filename to write to (omit for stdout)\n```\n\nYou can also import this into your own project and use any of the functions\nin the DOCS\n\n<!-- omit in toc -->\n## Table of Contents\n- [Example Use](#example-use)\n\t- [Help](#help)\n- [Changelog](#changelog)\n- [Install With PIP](#install-with-pip)\n- [Language information](#language-information)\n\t- [Built for](#built-for)\n- [Install Python on Windows](#install-python-on-windows)\n\t- [Chocolatey](#chocolatey)\n\t- [Download](#download)\n- [Install Python on Linux](#install-python-on-linux)\n\t- [Apt](#apt)\n- [How to run](#how-to-run)\n\t- [With VSCode](#with-vscode)\n\t- [From the Terminal](#from-the-terminal)\n- [Community Files](#community-files)\n\t- [Licence](#licence)\n\t- [Changelog](#changelog-1)\n\t- [Code of Conduct](#code-of-conduct)\n\t- [Contributing](#contributing)\n\t- [Security](#security)\n\t- [Support](#support)\n\n## Changelog\nSee the [CHANGELOG](/CHANGELOG.md) for more information.\n\n## Install With PIP\n\n```python\npip install licensecheck\n```\n\nHead to https://pypi.org/project/LicenseCheck/ for more info\n\n## Language information\n### Built for\nThis program has been written for Python 3 and has been tested with\nPython version 3.9.0 <https://www.python.org/downloads/release/python-380/>.\n\n## Install Python on Windows\n### Chocolatey\n```powershell\nchoco install python\n```\n### Download\nTo install Python, go to <https://www.python.org/> and download the latest\nversion.\n\n## Install Python on Linux\n### Apt\n```bash\nsudo apt install python3.9\n```\n\n## How to run\n### With VSCode\n1. Open the .py file in vscode\n2. Ensure a python 3.9 interpreter is selected (Ctrl+Shift+P > Python:Select\nInterpreter > Python 3.9)\n3. Run by pressing Ctrl+F5 (if you are prompted to install any modules, accept)\n### From the Terminal\n```bash\n./[file].py\n```\n\n## Community Files\n### Licence\nMIT License\nCopyright (c) FredHappyface\n(See the [LICENSE](/LICENSE.md) for more information.)\n\n### Changelog\nSee the [Changelog](/CHANGELOG.md) for more information.\n\n### Code of Conduct\nIn the interest of fostering an open and welcoming environment, we\nas contributors and maintainers pledge to make participation in our\nproject and our community a harassment-free experience for everyone.\nPlease see the\n[Code of Conduct](https://github.com/FHPythonUtils/.github/blob/master/CODE_OF_CONDUCT.md) for more information.\n\n### Contributing\nContributions are welcome, please see the [Contributing Guidelines](https://github.com/FHPythonUtils/.github/blob/master/CONTRIBUTING.md) for more information.\n\n### Security\nThank you for improving the security of the project, please see the [Security Policy](https://github.com/FHPythonUtils/.github/blob/master/SECURITY.md) for more information.\n\n### Support\nThank you for using this project, I hope it is of use to you. Please be aware that\nthose involved with the project often do so for fun along with other commitments\n(such as work, family, etc). Please see the [Support Policy](https://github.com/FHPythonUtils/.github/blob/master/SUPPORT.md) for more information.\n',
    'author': 'FredHappyface',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/FHPythonUtils/LicenseCheck',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



"""Define a foss compatability license_matrix

Standard disclaimer:: I am not a lawyer and there is no guarentee that the
information provided here is complete or correct. Do not take this as legal
advice on foss license compatability

https://en.wikipedia.org/wiki/IANAL


Types of license/ compatability

Public Domain
- Unlicense

Permissive Compatible
Permissive license compatible with gpl
- Mit
- Boost
- Bsd
- Isc
- Ncsa

Permissive Not Compatible
Permissive license NOT compatible with gpl
- Apache
- Eclipse
- Acedemic Free


Copyleft
permissive -> lgpl 2.1 -> gpl 2
permissive -> lgpl 3 -> gpl 3 -> agpl
permissive -> mpl -> gpl -> agpl (3 only)

permissive (any) -> EU
EU -> gpl -> agpl (3 only)
"""
from __future__ import annotations

from enum import Enum


class License(Enum):
	"""License Enum to hold a set of potential licenses
	"""
	# Public domain
	PUBLIC = 0
	UNLICENSE = 1
	# Permissive GPL compatible
	MIT = 10
	BOOST = 11
	BSD = 12
	ISC = 13
	NCSA = 14
	# Other permissive
	APACHE = 20
	ECLIPSE = 21
	ACEDEMIC_FREE = 22
	# LGPL
	LGPL_X = 30
	LGPL_2 = 31
	LGPL_3 = 32
	LGPL_2_PLUS = 33
	LGPL_3_PLUS = 34
	# GPL
	GPL_X = 40
	GPL_2 = 41
	GPL_3 = 42
	GPL_2_PLUS = 43
	GPL_3_PLUS = 44
	# AGPL
	AGPL_3_PLUS = 50
	# Other copyleft
	MPL = 60
	EU = 61

	# No License
	NO_LICENSE = 200

def licenseType(lice: str) -> list[License]:
	"""Return a list of license types from a license string

	Args:
		lice (str): license name

	Returns:
		list[License]: the license
	"""
	licenses = []
	liceL = lice.split(", ") # Deal with multilicense
	for liceS in liceL:
		lice = liceS.upper()
		if "PUBLIC DOMAIN" in lice:
			licenses.append(License.PUBLIC)
		elif "UNLICENSE" in lice:
			licenses.append(License.UNLICENSE)

		elif "MIT" in lice:
			licenses.append(License.MIT)
		elif "BOOST" in lice:
			licenses.append(License.BOOST)
		elif "BSD" in lice:
			licenses.append(License.BSD)
		elif "ISC" in lice:
			licenses.append(License.ISC)
		elif "NCSA" in lice:
			licenses.append(License.NCSA)

		elif "APACHE" in lice:
			licenses.append(License.APACHE)
		elif "ECLIPSE" in lice:
			licenses.append(License.ECLIPSE)
		elif "AFL" in lice:
			licenses.append(License.ACEDEMIC_FREE)

		elif "LGPL" in lice:
			if "LGPLV2+" in lice:
				licenses.append(License.LGPL_2_PLUS)
			elif "LGPLV3+" in lice:
				licenses.append(License.LGPL_3_PLUS)
			elif "LGPLV2+" in lice:
				licenses.append(License.LGPL_2_PLUS)
			elif "LGPLV3+" in lice:
				licenses.append(License.LGPL_3_PLUS)
			else:
				licenses.append(License.LGPL_X)
		elif "AGPL" in lice:
			licenses.append(License.AGPL_3_PLUS)
		elif "GPL" in lice:
			if "GPLV2+" in lice:
				licenses.append(License.GPL_2_PLUS)
			elif "GPLV3+" in lice:
				licenses.append(License.GPL_3_PLUS)
			elif "GPLV2+" in lice:
				licenses.append(License.GPL_2_PLUS)
			elif "GPLV3+" in lice:
				licenses.append(License.GPL_3_PLUS)
			else:
				licenses.append(License.GPL_X)

		elif "MPL" in lice:
			licenses.append(License.MPL)
		elif "EUPL" in lice:
			licenses.append(License.EU)
		else:
			licenses.append(License.NO_LICENSE)
	return licenses

# Permissive licenses compatible with GPL
PERMISSIVE = [License.MIT, License.BOOST, License.BSD, License.ISC, License.NCSA]
# Permissive licenses NOT compatible with GPL
PERMISSIVE_OTHER = [License.APACHE, License.ECLIPSE, License.ACEDEMIC_FREE]
# LGPL licenses
LGPL = [License.LGPL_2, License.LGPL_3, License.LGPL_2_PLUS, License.LGPL_3_PLUS, License.LGPL_X]
# GPL licenses (including AGPL)
GPL = [License.GPL_2, License.GPL_3, License.GPL_2_PLUS, License.GPL_3_PLUS,
License.GPL_X, License.AGPL_3_PLUS]
# Other Copyleft licenses
OTHER_COPYLEFT = [License.MPL, License.EU]

# Basic compat matrix
UNLICENSEE_INCOMPATIBLE = (PERMISSIVE + PERMISSIVE_OTHER + GPL +
LGPL + OTHER_COPYLEFT + [License.NO_LICENSE])
PERMISSIVE_INCOMPATIBLE = GPL + [License.EU, License.NO_LICENSE]
LGPL_INCOMPATIBLE = GPL + OTHER_COPYLEFT + PERMISSIVE_OTHER + [License.NO_LICENSE]
GPL_INCOMPATIBLE = PERMISSIVE_OTHER + [License.AGPL_3_PLUS, License.NO_LICENSE]
PERMISSIVE_GPL_INCOMPATIBLE = PERMISSIVE_OTHER + [License.NO_LICENSE]

# GPL compat matrix
# https://www.gnu.org/licenses/gpl-faq.html#AllCompatibility
GPL_2_INCOMPATIBLE = [License.GPL_3, License.GPL_3_PLUS, License.LGPL_3, License.LGPL_3_PLUS]
L_GPL_3_INCOMPATIBLE = [License.GPL_2]

def depCompatibleLice(myLicense: License, depLice: list[License]) -> bool:
	"""Identify if the end user license is compatible with the dependency
	license(s)

	Args:
		myLicense (License): end user license to check
		depLice (list[License]): dependency license

	Returns:
		bool: True if compatible, otherwise False
	"""
	blacklist = {
	License.UNLICENSE: UNLICENSEE_INCOMPATIBLE,
	License.PUBLIC: UNLICENSEE_INCOMPATIBLE,

	License.MIT: PERMISSIVE_INCOMPATIBLE,
	License.BOOST: PERMISSIVE_INCOMPATIBLE,
	License.BSD: PERMISSIVE_INCOMPATIBLE,
	License.ISC: PERMISSIVE_INCOMPATIBLE,
	License.NCSA: PERMISSIVE_INCOMPATIBLE,
	License.APACHE: PERMISSIVE_INCOMPATIBLE,
	License.ECLIPSE: PERMISSIVE_INCOMPATIBLE,
	License.ACEDEMIC_FREE: PERMISSIVE_INCOMPATIBLE,

	License.LGPL_X: LGPL_INCOMPATIBLE,
	License.LGPL_2: LGPL_INCOMPATIBLE,
	License.LGPL_3: LGPL_INCOMPATIBLE + L_GPL_3_INCOMPATIBLE,
	License.LGPL_2_PLUS: LGPL_INCOMPATIBLE,
	License.LGPL_3_PLUS: LGPL_INCOMPATIBLE + L_GPL_3_INCOMPATIBLE,
	License.GPL_X: GPL_INCOMPATIBLE,
	License.GPL_2: GPL_INCOMPATIBLE + GPL_2_INCOMPATIBLE,
	License.GPL_3: GPL_INCOMPATIBLE + L_GPL_3_INCOMPATIBLE,
	License.GPL_2_PLUS: GPL_INCOMPATIBLE,
	License.GPL_3_PLUS: GPL_INCOMPATIBLE + L_GPL_3_INCOMPATIBLE,
	License.AGPL_3_PLUS: PERMISSIVE_GPL_INCOMPATIBLE,

	License.MPL: LGPL + GPL + [License.EU],
	License.EU: PERMISSIVE_GPL_INCOMPATIBLE + LGPL + GPL + [License.MPL],
	} # yapf: disable

	blacklistResolved = blacklist[myLicense]
	for lice in depLice:
		if lice in blacklistResolved:
			return False
	return True
